/* All changes since Feb 12, 2002 by Moshe Bar
 * Copyright (C) 2000, 2001, Amnon Barak (amnon@cs.huji.ac.il)
 * Some sections copyright 2002 by Moshe Bar
 * Permission to use this software is hereby granted under the terms of the
 * GNU General Public License, as published by the Free Software Foundation.
 *
 * Author(s): Amnon Shiloh, Moshe Bar
 */

#include <linux/sched.h>
#ifdef CONFIG_MOSIX_FS
#include <asm/uaccess.h>
#include <linux/slab.h>
#include <linux/mfs.h>

void
pack_credits(struct mfs_cred *cred)
{
	register struct task_struct *p = current;
	register int i;

	cred->uid = p->uid;
	cred->euid = p->euid;
	cred->sgid = p->sgid;
	cred->fsuid = p->fsuid;
	cred->gid = p->gid;
	cred->egid = p->egid;
	cred->sgid = p->sgid;
	cred->fsgid = p->fsgid;
	cred->ngroups = p->ngroups;
	for(i = cred->ngroups-1 ; i >= 0 ; i--)
		cred->groups[i] = p->groups[i];
	cred->caps = p->cap_effective;
	cred->whereami = current->mosix.whereami ? : PE;
	cred->deppe = current->mosix.deppe ? : PE;
}

/* originally by Amnon Shiloh with changes by Moshe Bar*/
int
mfs_client_statfs(int pe, mfs_handle_t handle, struct statfs *statbuf)
{
        struct mfs_request_statfs s;
        struct mfs_reply_statfs *r;
        struct mfs_client_contact *con;
	int err;
	pack_credits(&s.cred);
	s.handle = handle;
	con = mfs_new_request(pe, MFS_REQUEST_STATFS, &s, sizeof(s), NULL, 0,NULL, 0);
	if(IS_ERR(con))
		return((int)con);
	err = mfs_client_receive(con, (void **)&r, NULL, NULL);
	if(err == MFS_REPLY_STATFS)
	{
		err = r->result;
		if(err >= 0)
		*statbuf = r->stat;
	}
	else if(err >= 0)
	{
	printk("openMosix: mfs_client_statfs: ambiguous reply %d\n", err);
	err = -EINVAL;
	}
	if(r)
	 kfree(r);
	mfs_release_contact(con);
	return(err);
}

inline void
mfs_file_up_to_date(struct file *filp)
{
	struct dentry *dp;
	struct mfs_dinfo *i;

	if((dp = filp->f_dentry) && (i = (struct mfs_dinfo *)dp->d_fsdata))
		i->latest = jiffies;
}

int
mfs_client_attach_handle(int pe, mfs_handle_t handle)
{
	struct mfs_request_attach_handle s;
	struct mfs_reply_attach_handle *r;
	struct mfs_client_contact *con;
	int err;

	s.handle = handle;
	con = mfs_new_request(pe, MFS_REQUEST_ATTACH_HANDLE, &s, sizeof(s),
							NULL, 0, NULL, 0);
	if(IS_ERR(con))
		return((int)con);
	err = mfs_client_receive(con, (void **)&r, NULL, NULL);
	if(err == MFS_REPLY_ATTACH_HANDLE)
		err = r->result;
	else if(err >= 0)
	{
		printk("mfs_client_attach_handle: unexpected reply %d\n", err);
		err = -EINVAL;
	}
	if(r)
		kfree(r);
	mfs_release_contact(con);
	return(err);
}

int
mfs_client_touch_handle(int pe, mfs_handle_t handle)
{
	struct mfs_request_touch_handle s;
	struct mfs_reply_touch_handle *r;
	struct mfs_client_contact *con;
	int err;

	s.handle = handle;
	con = mfs_new_request(pe, MFS_REQUEST_TOUCH_HANDLE, &s, sizeof(s),
							NULL, 0, NULL, 0);
	if(IS_ERR(con))
		return((int)con);
	err = mfs_client_receive(con, (void **)&r, NULL, NULL);
	if(err == MFS_REPLY_TOUCH_HANDLE)
		err = r->result;
	else if(err >= 0)
	{
		printk("mfs_client_touch_handle: unexpected reply %d\n", err);
		err = -EINVAL;
	}
	if(r)
		kfree(r);
	mfs_release_contact(con);
	return(err);
}

void
mfs_client_dispose_handle(int pe, mfs_handle_t handle)
{
	struct mfs_request_dispose_handle s;
	struct mfs_client_contact *con;
	void *r;

	s.handle = handle;
	con = mfs_new_request(pe, MFS_REQUEST_DISPOSE_HANDLE, &s, sizeof(s),
							NULL, 0, NULL, 0);
	if(IS_ERR(con))
		return;
	mfs_client_receive(con, &r, NULL, NULL);
	if(r)
		kfree(r);
	mfs_release_contact(con);
}

int
mfs_client_llseek(int pe, mfs_handle_t handle, struct file *filp, loff_t offset,
								int origin)
{
	struct mfs_request_llseek s;
	struct mfs_reply_llseek *r;
	struct mfs_client_contact *con;
	int err;

	pack_credits(&s.cred);
	s.handle = handle;
	s.file = *filp;
	s.offset = offset;
	s.origin = origin;
	con = mfs_new_request(pe, MFS_REQUEST_LLSEEK, &s, sizeof(s), NULL, 0,
								NULL, 0);
	if(IS_ERR(con))
		return((int)con);
	err = mfs_client_receive(con, (void **)&r, NULL, NULL);
	if(err == MFS_REPLY_LLSEEK)
	{
		cp_file_fields(filp, &r->file);
		err = r->result;
		if(err >= 0)
			mfs_file_up_to_date(filp);
	}
	else if(err >= 0)
	{
		printk("mfs_client_llseek: unexpected reply %d\n", err);
		err = -EINVAL;
	}
	if(r)
		kfree(r);
	mfs_release_contact(con);
	return(err);
}

ssize_t
mfs_client_read(int pe, mfs_handle_t handle, struct file *filp, char *buf,
						size_t count, loff_t *ppos)
{
	struct mfs_request_read s;
	struct mfs_reply_read *r;
	struct mfs_client_contact *con;
	void *data;
	int err;
	ssize_t i, result = 0;
	int user_err = 0;
	unsigned int done = 0;

	if((int)count < 0)
		return(-EINVAL);
	pack_credits(&s.cred);
	s.handle = handle;
	s.file = *filp;
	s.count = count;
	s.pos = *ppos;
	con = mfs_new_request(pe, MFS_REQUEST_READ, &s, sizeof(s), NULL, 0,
								NULL, 0);
	if(IS_ERR(con))
		return((int)con);
	while(!done)
	{
		err = mfs_client_receive(con, (void **)&r, &data, NULL);
		if(err < 0)
		{
			result = err;
			break;
		}
		switch(err)
		{
			case MFS_INTERIM_DATA:
				i = ((struct mfs_interim_data *)r)->count;
				if(!user_err && copy_to_user(buf, data, i))
					user_err = -EFAULT;
				else
					buf += i;
				/* do not exit: it is always the server
				 * that should close the connection
				 */
				break;
			case MFS_REPLY_READ:
				done = 1;
				if(!user_err && r->datalen &&
					copy_to_user(buf, data, r->datalen))
					user_err = -EFAULT;
				cp_file_fields(filp, &r->file);
				*ppos = r->ppos;
				result = r->result;
				if(result >= 0)
					mfs_file_up_to_date(filp);
				break;
			default:
				printk("mfs_client_read: unexpected reply %d\n",
									err);
				result = -EINVAL;
				done = 1;
				break;
				
		}
		if(r)
			kfree(r);
	}
	mfs_release_contact(con);
	return(user_err ? : result);
}

ssize_t
mfs_client_write(int pe, mfs_handle_t handle, struct file *filp, char *buf,
					size_t count, loff_t *ppos, long flim)
{
	struct mfs_request_write s;
	struct mfs_interim_data i;
	struct mfs_reply_write *r;
	struct mfs_client_contact *con;
	void *data;
	int err;
	ssize_t result = 0;
	int user_err = 0;
	size_t max, sent, cnt;
	char smallbuf[4];
	char *tbuf = NULL;	/* NULL just because bad compiler complains */

	if(count == 0)
		return(0);
	if((int)count < 0)
		return(-EINVAL);
	pack_credits(&s.cred);
	s.handle = handle;
	s.file = *filp;
	s.count = count;
	s.pos = *ppos;
	s.flim = flim;
	for(max = count > 16384 ? 16384 : count ; max > sizeof(smallbuf) ;
								max >>= 1)
	if((tbuf = (char *)kmalloc(max, GFP_KERNEL)))
		break;
	if(max <= sizeof(smallbuf))
	{
		tbuf = smallbuf;
		max = sizeof(smallbuf);
	}
	s.datalen = sent = count < max ? count : max;
	if(copy_from_user(tbuf, buf, sent))
	{
		user_err = -EFAULT;
		goto free_buf;
	}
	buf += sent;
	con = mfs_new_request(pe, MFS_REQUEST_WRITE, &s, sizeof(s), tbuf, sent,
								NULL, 0);
	if(IS_ERR(con))
	{
		result = (int)con;
		goto free_buf;
	}
	while(sent < count)
	{
		if(mfs_client_contact_has_data(con))
			break;
		cnt = count - sent;
		if(cnt > max)
			cnt = max;
		if(user_err || copy_from_user(tbuf, buf, cnt))
		{
			user_err = -EFAULT;
			mfs_client_send(con, MFS_STOP, NULL, 0,
					NULL, 0, NULL, 0);
			break;	/* let the server quit first */
		}
		i.count = cnt;
		if((err = mfs_client_send(con, MFS_INTERIM_DATA,
			&i, sizeof(i), (void *)tbuf, cnt,
			NULL, 0)))
		{
			result = err;
			break;
		}
		buf += cnt;
		sent += cnt;
	}
	err = mfs_client_receive(con, (void **)&r, &data, NULL);
	if(err == MFS_CONTINUE)
		err = mfs_client_receive(con, (void **)&r, &data, NULL);
	if(err == MFS_REPLY_WRITE)
	{
		cp_file_fields(filp, &r->file);
		*ppos = r->ppos;
		result = r->result;
		if(result >= 0)
			mfs_file_up_to_date(filp);
		if(r->hadsigxfsz)
			send_sig(SIGXFSZ, current, 0);
	}
	else if(err < 0)
		result = err;
	else
	{
		printk("mfs_client_write: unexpected reply %d\n", err);
		result = -EINVAL;
	}
	if(r)
		kfree(r);
	mfs_release_contact(con);
	free_buf:
	if(max != sizeof(smallbuf))
		kfree(tbuf);
	return(user_err ? : result);
}

int
mfs_client_readdir_transfer(void *d, int len, void *dirent, filldir_t filldir)
{
	ino_t ino;
	loff_t off;
	unsigned int type;
	short namelen;
	int err;

	while(len)
	{
		len -= sizeof(ino)+sizeof(off)+sizeof(type)+sizeof(namelen);
		if(len < 0)
			return(-EINVAL);
		ino = *((ino_t *)d)++;
		off = *((loff_t *)d)++;
		type = *((unsigned int *)d)++;
		namelen = *((short *)d)++;
		if(len < namelen)
			return(-EINVAL);
		len -= namelen;
		if((err = filldir(dirent, (const char *)d, namelen, off, ino,
									type)))
			return(err);
		d += namelen;
	}
	return(0);
}

int
mfs_client_readdir(int pe, mfs_handle_t handle, struct file *filp, void *dirent,
	filldir_t filldir, int policy, int maxbytes, int *packets, int *bytes)
{
	struct mfs_request_readdir s;
	struct mfs_reply_readdir *r;
	struct mfs_interim_readdir ri;
	struct mfs_client_contact *con;
	void *data;
	ssize_t i;
	int result = 0;
	unsigned int done = 0;

	pack_credits(&s.cred);
	s.handle = handle;
	s.file = *filp;
	s.policy = policy;
	s.maxbytes = maxbytes;
	*packets = *bytes = 0;
	con = mfs_new_request(pe, MFS_REQUEST_READDIR, &s, sizeof(s), NULL, 0,
								NULL, 0);
	if(IS_ERR(con))
		return((int)con);
	while(!done && (result =
		mfs_client_receive(con, (void **)&r, &data, NULL)) >= 0)
	{
		switch(result)
		{
			case MFS_INTERIM_DATA:
				i = ((struct mfs_interim_data *)r)->count;
				(*packets)++;
				(*bytes) += i;
				ri.error = mfs_client_readdir_transfer(data, i,
							dirent, filldir);
				if((result = mfs_client_send(con,
					MFS_INTERIM_READDIR, &ri, sizeof(ri),
					NULL, 0, NULL, 0)))
					done = 1;
				break;
			case MFS_REPLY_READDIR:
				done = 1;
				(*bytes) += r->datalen;
				if(!r->datalen ||
					!(result = mfs_client_readdir_transfer(
					data, r->datalen, dirent, filldir)))
					result = r->result;
				cp_file_fields(filp, &r->file);
				if(result >= 0)
					mfs_file_up_to_date(filp);
				break;
			default:
				printk("mfs_client_readdir: unexpected reply %d\n", result);
				result = -EINVAL;
				done = 1;
				break;
		}
		if(r)
			kfree(r);
	}
	mfs_release_contact(con);
	return(result);
}

int
mfs_client_ioctl(int pe, mfs_handle_t handle, struct file *filp,
		unsigned int cmd, unsigned long arg, int size, int *data)
{
	struct mfs_request_ioctl s;
	struct mfs_reply_ioctl *r;
	struct mfs_client_contact *con;
	int err;

	pack_credits(&s.cred);
	s.handle = handle;
	s.file = *filp;
	s.cmd = cmd;
	s.arg = arg;
	s.data = *data;
	s.size = size;
	con = mfs_new_request(pe, MFS_REQUEST_IOCTL, &s, sizeof(s), NULL, 0,
								NULL, 0);
	if(IS_ERR(con))
		return((int)con);
	err = mfs_client_receive(con, (void **)&r, NULL, NULL);
	if(err == MFS_REPLY_IOCTL)
	{
		cp_file_fields(filp, &r->file);
		if(size < 0)
			*data = r->data;
		err = r->result;
		if(err >= 0)
			mfs_file_up_to_date(filp);
	}
	else if(err >= 0)
	{
		printk("mfs_client_ioctl: unexpected reply %d\n", err);
		err = -EINVAL;
	}
	if(r)
		kfree(r);
	mfs_release_contact(con);
	return(err);
}

int
mfs_client_fsync(int pe, mfs_handle_t handle, struct file *filp, int datasync)
{
	struct mfs_request_fsync s;
	struct mfs_reply_fsync *r;
	struct mfs_client_contact *con;
	int err;

	pack_credits(&s.cred);
	s.handle = handle;
	s.file = *filp;
	s.datasync = datasync;
	con = mfs_new_request(pe, MFS_REQUEST_FSYNC, &s, sizeof(s), NULL, 0,
								NULL, 0);
	if(IS_ERR(con))
		return((int)con);
	err = mfs_client_receive(con, (void **)&r, NULL, NULL);
	if(err == MFS_REPLY_FSYNC)
	{
		cp_file_fields(filp, &r->file);
		err = r->result;
		if(err >= 0)
			mfs_file_up_to_date(filp);
	}
	else if(err >= 0)
	{
		printk("mfs_client_fsync: unexpected reply %d\n", err);
		err = -EINVAL;
	}
	if(r)
		kfree(r);
	mfs_release_contact(con);
	return(err);
}

int
mfs_client_open(int pe, mfs_handle_t handle, struct inode *inode, struct file *filp)
{
	struct mfs_client_contact *con;
	struct mfs_request_open s;
	struct mfs_reply_open *r;
	int ret;

	pack_credits(&s.cred);
	s.handle = handle;
	s.file = *filp;
	s.inode = *inode;
	con = mfs_new_request(pe, MFS_REQUEST_OPEN, &s, sizeof(s), NULL, 0,
								NULL, 0);
	if (IS_ERR(con))
		return ((int) con);

	ret = mfs_client_receive(con, (void **) &r, 0, 0);
	switch (ret)
	{
		case MFS_REPLY_OPEN:
			cp_file_fields(filp, &r->file);
			ret = r->result;
			if (ret >= 0)
				mfs_file_up_to_date(filp);
			break;
		default:
			printk("mfs_client_open: unexpected reply %d\n", ret);
			ret = -EINVAL;
			break;
	}
	mfs_release_contact(con);
	return ret;
}

int
mfs_client_close(int pe, mfs_handle_t handle, struct inode *inode, struct file *filp)
{
	struct mfs_client_contact *con;
	struct mfs_request_open s;
	struct mfs_reply_open *r;
	int ret;

	pack_credits(&s.cred);
	s.handle = handle;
	s.file = *filp;
	s.inode = *inode;
	con = mfs_new_request(pe, MFS_REQUEST_CLOSE, &s, sizeof(s), NULL, 0,
								NULL, 0);
	if (IS_ERR(con))
		return ((int) con);

	ret = mfs_client_receive(con, (void **) &r, 0, 0);
	switch (ret)
	{
		case MFS_REPLY_CLOSE:
			cp_file_fields(filp, &r->file);
			ret = r->result;
			if (ret >= 0)
				mfs_file_up_to_date(filp);
			break;
		default:
			printk("mfs_client_closedir: unexpected reply %d\n", ret);
			ret = -EINVAL;
			break;
	}
	mfs_release_contact(con);
	return ret;
}

int
mfs_client_revalidate(int pe, mfs_handle_t handle, struct subinode *iinfo)
{
	struct mfs_request_revalidate s;
	struct mfs_reply_revalidate *r;
	struct mfs_client_contact *con;
	int err;

	pack_credits(&s.cred);
	s.handle = handle;
	con = mfs_new_request(pe, MFS_REQUEST_REVALIDATE, &s, sizeof(s),
							NULL, 0, NULL, 0);
	if(IS_ERR(con))
		return((int)con);
	err = mfs_client_receive(con, (void **)&r, NULL, NULL);
	if(err == MFS_REPLY_REVALIDATE)
	{
		*iinfo = r->iinfo;
		err = r->result;
	}
	else if(err >= 0)
	{
		printk("mfs_client_revalidate: unexpected reply %d\n", err);
		err = -EINVAL;
	}
	if(r)
		kfree(r);
	mfs_release_contact(con);
	return(err);
}

int
mfs_client_express_lookup(int pe, mfs_handle_t *handle, char *name,
	unsigned int follow, struct subinode *iinfo, char **newname,
	struct mfs_completion *complete)
{
	struct mfs_request_express s;
	struct mfs_reply_express *r;
	struct mfs_client_contact *con;
	int err;
	void *name_back, *more_data;

	*newname = NULL;
	pack_credits(&s.cred);
	s.handle = *handle;
	s.namelen = strlen(name) + 1;
	s.follow = follow;
	s.link_count = current->link_count;
	s.total_link_count = current->total_link_count;
	s.complete = *complete;
	con = mfs_new_request(pe, MFS_REQUEST_EXPRESS, &s, sizeof(s),
				(void *)name, s.namelen,
				s.complete.more_data, s.complete.more_data_len);
	if(IS_ERR(con))
		return((int)con);
	err = mfs_client_receive(con, (void **)&r, &name_back, &more_data);
	if(err == MFS_REPLY_EXPRESS)
	{
		*handle = r->handle;
		*iinfo = r->iinfo;
		*complete = r->complete;
		err = r->result;
		if(!err)
		{
			if(!r->namelen)
				*newname = NULL;
			else if((*newname = kmalloc(r->namelen, GFP_KERNEL)))
				memcpy(*newname, name_back, r->namelen);
			else
				err = -ENOMEM;
		}
		if(complete->more_data)
		{
			if((complete->more_data =
				kmalloc(complete->more_data_len, GFP_KERNEL)))
				memcpy(complete->more_data, more_data,
						complete->more_data_len);
			else
			{
				if(!err)
					err = -ENOMEM;
				if(*newname)
				{
					kfree(*newname);
					*newname = NULL;
				}
			}
		}
	}
	else if(err >= 0)
	{
		printk("mfs_client_express_lookup: unexpected reply %d\n", err);
		err = -EINVAL;
	}
	if(r)
		kfree(r);
	mfs_release_contact(con);
	return(err);
}

char *
mfs_client_check_path(int pe, mfs_handle_t handle)
{
	struct mfs_request_checkpath s;
	struct mfs_reply_checkpath *r;
	char *path;
	char *copy = NULL;
	struct mfs_client_contact *con;
	int err;

	pack_credits(&s.cred);
	s.handle = handle;
	con = mfs_new_request(pe, MFS_REQUEST_CHECKPATH, &s, sizeof(s), NULL, 0,
								NULL, 0);
	if(IS_ERR(con))
		return(NULL);
	err = mfs_client_receive(con, (void **)&r, (void *)&path, NULL);
	if(err == MFS_REPLY_CHECKPATH)
	{
		if(r->namelen && (copy = kmalloc(r->namelen, GFP_KERNEL)))
			memcpy(copy, path, r->namelen);
	}
	else if(err >= 0)
	{
		printk("mfs_client_check_path: unexpected reply %d\n", err);
		err = -EINVAL;
	}
	if(r)
		kfree(r);
	mfs_release_contact(con);
	return(copy);
}

int
mfs_client_setattr(int pe, mfs_handle_t handle, struct iattr *iattr)
{
	struct mfs_request_setattr s;
	struct mfs_reply_setattr *r;
	struct mfs_client_contact *con;
	int err;

	pack_credits(&s.cred);
	s.handle = handle;
	s.attr = *iattr;
	con = mfs_new_request(pe, MFS_REQUEST_SETATTR, &s, sizeof(s),
							NULL, 0, NULL, 0);
	if(IS_ERR(con))
		return((int)con);
	err = mfs_client_receive(con, (void **)&r, NULL, NULL);
	if(err == MFS_REPLY_SETATTR)
		err = r->result;
	else if(err >= 0)
	{
		printk("mfs_client_setattr: unexpected reply %d\n", err);
		err = -EINVAL;
	}
	if(r)
		kfree(r);
	mfs_release_contact(con);
	return(err);
}

#endif /* CONFIG_MOSIX_FS */
