/* All changes since Feb 12, 2002 copyright by Moshe Bar (moshe@moshebar.com)
 * Copyright (C) 2000, 2001, Amnon Barak (amnon@cs.huji.ac.il)
 *
 * Permission to use this software is hereby granted under the terms of the
 * GNU General Public License, as published by the Free Software Foundation.
 *
 * THIS  SOFTWARE  IS  PROVIDED IN ITS  "AS IS"  CONDITION, WITH NO WARRANTY
 * WHATSOEVER. NO LIABILITY OF ANY KIND FOR ANY DAMAGES WHATSOEVER RESULTING
 * FROM THE USE OF THIS SOFTWARE WILL BE ACCEPTED.
 */
/*
 * Author(s): Amnon Shiloh, Moshe Bar
 */


#include <linux/sched.h>
#ifdef CONFIG_MOSIX_FS
#include <linux/hpc.h>
#include <linux/hpcctl.h>
#include <asm/uaccess.h>
#include <linux/smp_lock.h>
#include <linux/slab.h>
#include <linux/fs.h>
#include <linux/mfs.h>
#include <linux/mfs_socket.h>
#include <linux/locks.h>
#include <linux/module.h>
#include <hpc/routines.h>
#include <hpc/defs.h>
#include <linux/dcache.h>
#include <linux/quotaops.h>
#include <linux/ext2_fs.h>
#include <linux/file.h>

int MFS_PE;
int mfs_is_mounted;
char mfs_is_ro;
#ifdef CONFIG_MOSIX_DFSA
char mfs_is_dfsa;
#include <linux/dfsa_interface.h>
#endif /* CONFIG_MOSIX_DFSA */
#define	mfs_loopback_test	0
int mfs_debug;
struct super_block *mfs_super;
extern int may_create(struct inode *, struct dentry *);

#define	AN_IMPOSSIBLE_ERROR	(-ECHILD)	/* cannot occur in FS context */

ino_t
global_inum(struct inode *ip)
{
	dev_t dev = ip->i_dev;
	ino_t mask = (MFS_PE << 16) ^ (MAJOR(dev) << 16) ^ (MINOR(dev) << 24);
	ino_t result = ip->i_ino ^ mask;

	if(ip->i_ino == mfs_root_ino && dev == mfs_root_dev)
		return(MFS_PE);
	if(((unsigned long)result) <= MFS_ROOT_INO)
		result ^= 0x80800000;
	return(result);
}

inline void
mfs_get_handling(struct dentry *dp, int *where, mfs_handle_t *handle)
{
	struct mfs_dinfo *p = (struct mfs_dinfo *)dp->d_fsdata;

	*where = p->pe;
	*handle = p->handle;
}

inline int
mfs_is_last_use(struct dentry *dp, int *where, mfs_handle_t *handle)
{
	if(dp->d_fsdata)
	{
		*where = ((struct mfs_dinfo *)dp->d_fsdata)->pe;
		*handle = ((struct mfs_dinfo *)dp->d_fsdata)->handle;
		if(atomic_read(&dp->d_count) > 1)
			return(0);
		kfree(dp->d_fsdata);
		dp->d_fsdata = NULL;
		return(1);
	}
	else
	{
		*where = 0;
		*handle = MFS_ROOT_INO;
		return(0);
	}
}

inline int
mfs_low_attach_handle(mfs_handle_t handle)
{
	if(handle > MFS_ROOT_INO)	/* do not waste time on root */
		return(mfs_get_handle(handle));
	return(0);
}

inline int
mfs_attach_handle(int pe, mfs_handle_t handle)
{
	if(handle <= MFS_ROOT_INO)
		return(0);
	mfs_count_attach_handle(pe);
	if(pe == MFS_PE && !mfs_loopback_test)
		return(mfs_low_attach_handle(handle));
	else if(pe)
		return(mfs_client_attach_handle(pe, handle));
	return(0);
}

inline int
mfs_low_touch_handle(mfs_handle_t handle)
{
	if(handle > MFS_ROOT_INO)	/* do not waste time on root */
		return(mfs_access_handle(handle));
	return(0);
}

inline int
mfs_touch_handle(int pe, mfs_handle_t handle)
{
	if(handle <= MFS_ROOT_INO)
		return(0);
	mfs_count_touch_handle(pe);
	if(pe == MFS_PE && !mfs_loopback_test)
		return(mfs_low_touch_handle(handle));
	else if(pe)
		return(mfs_client_touch_handle(pe, handle));
	return(0);
}

inline void
mfs_low_dispose_handle(mfs_handle_t handle)
{
	if(handle > MFS_ROOT_INO)	/* do not waste time on root */
		mfs_put_handle(handle);
}

inline void
mfs_dispose_handle(int pe, mfs_handle_t handle)
{
	if(handle <= MFS_ROOT_INO)
		return;
	mfs_count_dispose_handle(pe);
	if(pe == MFS_PE && !mfs_loopback_test)
		mfs_low_dispose_handle(handle);
	else if(pe)
		mfs_client_dispose_handle(pe, handle);
}

extern struct file_operations mfs_file_file_operations;
extern struct file_operations mfs_symlink_file_operations;
extern struct file_operations mfs_dir_file_operations;
extern struct inode_operations mfs_file_inode_operations;
extern struct inode_operations mfs_symlink_inode_operations;
extern struct inode_operations mfs_dir_inode_operations;
extern struct dentry_operations mfs_dentry_operations;

void
mfs_ip_to_subip(struct subinode *subip, struct inode *ip)
{
	subip->si_mode = ip->i_mode;
	subip->si_nlink = ip->i_nlink;
	subip->si_iuid = ip->i_uid;
	subip->si_gid = ip->i_gid;
	subip->si_rdev = ip->i_rdev;
	subip->si_size = ip->i_size;
	subip->si_atime = ip->i_atime;
	subip->si_ctime = ip->i_ctime;
	subip->si_mtime = ip->i_mtime;
	subip->si_blksize = ip->i_blksize;
	subip->si_blocks = ip->i_blocks;
	subip->si_origino = ip->i_ino;
	subip->si_origdev = ip->i_dev;
}

void
mfs_subip_to_ip(struct inode *ip, struct subinode *subip)
{
	ip->i_mode = subip->si_mode;
	ip->i_nlink = subip->si_nlink;
	ip->i_uid = subip->si_iuid;
	ip->i_gid = subip->si_gid;
	ip->i_rdev = subip->si_rdev;
	ip->i_size = subip->si_size;
	ip->i_atime = subip->si_atime;
	ip->i_ctime = subip->si_ctime;
	ip->i_mtime = subip->si_mtime;
	ip->i_blksize = subip->si_blksize;
	ip->i_blocks = subip->si_blocks;
	ip->i_version = subip->si_origino;
	ip->i_generation = subip->si_origdev;
	ip->i_sb = mfs_super;
	switch(ip->i_mode & S_IFMT)
	{
		case S_IFREG:
			ip->i_op = &mfs_file_inode_operations;
			ip->i_fop = &mfs_file_file_operations;
			break;
		case S_IFLNK:
			ip->i_op = &mfs_symlink_inode_operations;
			ip->i_fop = &mfs_symlink_file_operations;
			break;
		case S_IFDIR:
			ip->i_op = &mfs_dir_inode_operations;
			ip->i_fop = &mfs_dir_file_operations;
			break;
		default:
			printk("MFS: inum %d of bad mode (0%o)\n",
						(int)ip->i_ino, ip->i_mode);
			make_bad_inode(ip);
			break;
	}
}

void
cp_file_fields(struct file *filto, struct file *filfrom)
{
	filto->f_mode = filfrom->f_mode;
	filto->f_pos = filfrom->f_pos;
	filto->f_owner = filfrom->f_owner;
	filto->f_reada = filfrom->f_reada;
	filto->f_ramax = filfrom->f_ramax;
	filto->f_raend = filfrom->f_raend;
	filto->f_ralen = filfrom->f_ralen;
	filto->f_rawin = filfrom->f_rawin;
/* the following two fields are not actually currently in use!
	filto->f_uid = filfrom->f_uid;
	filto->f_gid = filfrom->f_gid;
*/
	filto->f_error = filfrom->f_error;
	filto->f_version = filfrom->f_version;
	filto->private_data = filfrom->private_data;
}

int
prepare_dummy_file(mfs_handle_t handle, struct file *dummy, struct file *filp)
{
	struct nameidata nd;
	int err;

	if((err = mfs_to_local(handle, &nd)))
		return(err);
	if(nd.dentry->d_inode && nd.dentry->d_inode->i_op)
		dummy->f_op = nd.dentry->d_inode->i_fop;
	else
		dummy->f_op = NULL;
	dummy->f_dentry = nd.dentry;
	dummy->f_vfsmnt = nd.mnt;
	dummy->f_flags = filp->f_flags & (O_LARGEFILE|O_SYNC);
	dummy->f_iobuf = NULL;
	dummy->f_iobuf_lock = 0;
	atomic_set(&dummy->f_count, 1);
	cp_file_fields(dummy, filp);
	return(0);
}

void
collect_dummy_file(struct file *dummy, struct file *filp)
{
	mfs_lput(dummy->f_dentry, dummy->f_vfsmnt);
	cp_file_fields(filp, dummy);
}

loff_t
mfs_dir_llseek(struct file *filp, loff_t offset, int origin)
{
	int where;
	mfs_handle_t handle;

	mfs_get_handling(filp->f_dentry, &where, &handle);
	mfs_count_llseek(where);
	if(where != MFS_PE || mfs_loopback_test)
		return(mfs_client_llseek(where, handle, filp, offset, origin));
	return(mfs_low_llseek(handle, filp, offset, origin));
}

loff_t
mfs_file_llseek(struct file *filp, loff_t offset, int origin)
{
	uint64_t newoff;

	switch(origin)
	{
		case 0:
			newoff = offset;
			break;
		case 1:
			newoff = offset + filp->f_pos;
			break;
		default:
			goto must_call;
	}
	if(!(newoff >> 32))
	{
		if(newoff != filp->f_pos)
		{
			lock_kernel();
			filp->f_pos = newoff;
			filp->f_reada = 0;
			filp->f_version = ++event;
			unlock_kernel();
		}
		return(newoff);
	}

	must_call:
	return(mfs_dir_llseek(filp, offset, origin));
}

loff_t
mfs_low_llseek(mfs_handle_t handle, struct file *filp, loff_t offset,
								int origin)
{
	loff_t ret;
	struct file dummy;
	extern loff_t llseek(struct file *, loff_t, int);

	if((ret = prepare_dummy_file(handle, &dummy, filp)))
		return(ret);
	lock_kernel();
	ret = llseek(&dummy, offset, origin);
	unlock_kernel();
	collect_dummy_file(&dummy, filp);
	return(ret);
}

ssize_t
mfs_read(struct file *filp, char *buf, size_t count, loff_t *ppos)
{
	int where;
	mfs_handle_t handle;

	if(!(filp->f_mode & FMODE_READ))
		return(-EBADF);
	mfs_get_handling(filp->f_dentry, &where, &handle);
	mfs_count_read(where, count);
	if(where != MFS_PE || mfs_loopback_test)
		return(mfs_client_read(where, handle, filp, buf, count, ppos));
	return(mfs_low_read(handle, filp, buf, count, ppos));
}

ssize_t
mfs_low_read(mfs_handle_t handle, struct file *filp, char *buf, size_t count,
								loff_t *ppos)
{
	loff_t ret;
	struct file *dummy;
	struct file_operations *fops;
	loff_t mppos;

	current->mosix.dirty_bits |= MFSARG_EMPTYF_PRI;
	dummy = get_empty_filp();
	current->mosix.dirty_bits &= ~MFSARG_EMPTYF_PRI;
	if(!dummy)
		return(-ENFILE);

	if((ret = prepare_dummy_file(handle, dummy, filp)))
	{
		put_filp(dummy);
		return(ret);
	}
	dget(dummy->f_dentry);
	mntget(dummy->f_vfsmnt);
	fops = dummy->f_dentry->d_inode->i_fop;
	mppos = *ppos;
#ifdef ONE_DAY_S_FILES_WILL_BE_USED_FOR_RDONLY_FILES
	if(ip->i_sb)
		file_move(dummy, &ip->i_sb->s_files);
#endif /* ONE_DAY_S_FILES_WILL_BE_USED_FOR_RDONLY_FILES */
	if(!fops || !fops->read)
		ret = -EINVAL;
	else
		ret = fops->read(dummy, buf, count, &mppos);
	collect_dummy_file(dummy, filp);
	*ppos = mppos;
	dummy->f_mode &= ~FMODE_WRITE;
	fput(dummy);
	return(ret);
}

ssize_t
mfs_write(struct file *filp, const char *buf, size_t count, loff_t *ppos)
{
	int where;
	mfs_handle_t handle;
	int flim = current->rlim[RLIMIT_FSIZE].rlim_cur;

	if(!(filp->f_mode & FMODE_WRITE))
		return(-EBADF);
	mfs_get_handling(filp->f_dentry, &where, &handle);
	mfs_count_write(where, count);
	if(where != MFS_PE || mfs_loopback_test)
		return(mfs_client_write(where, handle, filp, (char *)buf, count,
								ppos, flim));
	return(mfs_low_write(handle, filp, (char *)buf, count, ppos, flim));
}

ssize_t
mfs_low_write(mfs_handle_t handle, struct file *filp, char *buf, size_t count,
							loff_t *ppos, long flim)
{
	loff_t ret;
	struct file *dummy;
	struct file_operations *fops;
	struct super_block *sb;
	struct inode *ip;
	loff_t mppos;

	/* unlike other functions, we need to allocate a dummy "file"
	 * in the proper way, because NFS files could live even after
	 * we exit */
	current->mosix.dirty_bits |= MFSARG_EMPTYF_PRI;
	dummy = get_empty_filp();
	current->mosix.dirty_bits &= ~MFSARG_EMPTYF_PRI;
	if(!dummy)
		return(-ENFILE);
	
	if((ret = prepare_dummy_file(handle, dummy, filp)))
	{
		put_filp(dummy);
		return(ret);
	}
	dget(dummy->f_dentry);
	mntget(dummy->f_vfsmnt);
	current->rlim[RLIMIT_FSIZE].rlim_cur = flim;
	ip = dummy->f_dentry->d_inode;
	sb = ip->i_sb;
	fops = ip->i_fop;
	mppos = *ppos;
	if(filp->f_flags & O_APPEND)
		mppos = ip->i_size;
	if(sb)
		file_move(dummy, &sb->s_files);
	if(get_write_access(ip))
		dummy->f_flags |= O_NOWRITEACCESS;
	if((ret = locks_verify_area(FLOCK_VERIFY_WRITE, ip, NULL, mppos,count)))
		;
	else if(sb && (sb->s_flags & MS_RDONLY))
		ret = -EROFS;
	else if(!fops || !fops->write)
		ret = -EINVAL;
	else
		ret = fops->write(dummy, buf, count, &mppos);
	collect_dummy_file(dummy, filp);
	*ppos = mppos;
	fput(dummy);
	return(ret);
}

int
mfs_open(struct inode *inode, struct file *filp)
{
	int where;
	mfs_handle_t handle;

	mfs_get_handling(filp->f_dentry, &where, &handle);
	if (where != MFS_PE)
		return mfs_client_open(where, handle, inode, filp);
	return mfs_low_open(handle, inode, filp);
}

int
mfs_close(struct inode *inode, struct file *filp)
{
	int where;
	mfs_handle_t handle;

	mfs_get_handling(filp->f_dentry, &where, &handle);
	if (where != MFS_PE)
		return mfs_client_close(where, handle, inode, filp);
	return mfs_low_close(handle, inode, filp);
}

int
mfs_readdir(struct file *filp, void *dirent, filldir_t filldir)
{
	int where;
	mfs_handle_t handle;
	struct task_struct *p = current;
	int policy;
	int packets, bytes;
	int result;

	switch(p->mosix.dirty_bits &
		(MFSARG_GETDENTS|MFSARG_GETDENTS64|MFSARG_OLDREADDIR))
	{
		case MFSARG_GETDENTS:
			policy = MFS_READDIR_GETDENTS;
			break;
		case MFSARG_GETDENTS64:
			policy = MFS_READDIR_GETDENTS64;
			break;
		case MFSARG_OLDREADDIR:
			policy = MFS_READDIR_JUSTONE;
			break;
		default:
			policy = MFS_READDIR_UNKNOWN;
			break;
	}
	mfs_get_handling(filp->f_dentry, &where, &handle);
	if(where != MFS_PE || mfs_loopback_test)
		result = mfs_client_readdir(where, handle, filp, dirent,
			filldir, policy, p->mosix.dirty_arg, &packets, &bytes);
	else
		result = mfs_low_readdir(handle, filp, dirent, filldir,
							&packets, &bytes);
	mfs_count_readdir(where, packets, bytes);
	return(result);
}

struct mfs_local_readdir_callback
{
	filldir_t realfilldir;
	void *realdirent;
	struct dentry *parent;
	struct vfsmount *pmnt;
	int packets, bytes, inbytes;
};

int
mfs_local_filldir(void *__cb, const char *name, int namelen, loff_t offset,
						ino_t ino, unsigned int type)
{
	struct mfs_local_readdir_callback *cb = __cb;
	struct inode ji;
	struct inode *ri;
	unsigned int b = sizeof(ino_t) + sizeof(loff_t) + sizeof(namelen) +
						strnlen(name, namelen) + 1;

	cb->bytes += b;
	if((cb->inbytes += b) > MFS_MAXBLOCK)
	{
		cb->packets++;
		cb->inbytes = b;
	}
	ji.i_ino = ino;
	ji.i_dev = cb->parent->d_inode->i_dev;
	/* to present a single, flat filesystem, we must adjust in case the
	 * son falls outside the parent's file-system. */
	if(namelen == 1 && name[0] == '.')
		;	/* nothing */
	else if(namelen == 2 && name[0] == '.' && name[1] == '.')
	{
		if(cb->parent == cb->pmnt->mnt_root)
		{
			ri = cb->pmnt->mnt_mountpoint->d_inode;
			ji.i_ino = ri->i_ino;
			ji.i_dev = ri->i_dev;
		}
	}
	else
	{
		struct list_head *p;
		struct vfsmount *m;

		spin_lock(&dcache_lock);
		for(p = cb->pmnt->mnt_mounts.next ; p != &cb->pmnt->mnt_mounts ;
			p = p->next)
		{
			m = list_entry(p, struct vfsmount, mnt_child);
			if(m->mnt_mountpoint->d_inode->i_ino == ji.i_ino)
			{
				struct inode *mroot = m->mnt_root->d_inode;

				ji.i_dev = mroot->i_dev;
				ji.i_ino = mroot->i_ino;
				break;
			}
		}
		spin_unlock(&dcache_lock);
	}
	return(cb->realfilldir(cb->realdirent, name, namelen, offset,
						global_inum(&ji), type));
}

int
mfs_low_open(mfs_handle_t handle, struct inode *inode, struct file *filp)
{
	struct file dummy;
	struct file_operations *fops;
	int ret;

	ret = prepare_dummy_file(handle, &dummy, filp);
	if (ret)
		return ret;
	fops = dummy.f_dentry->d_inode->i_fop;
	if (fops && fops->open)
		ret = fops->open(inode, &dummy);
	collect_dummy_file(&dummy, filp);
	return ret;
}

int
mfs_low_close(mfs_handle_t handle, struct inode *inode, struct file *filp)
{
	struct file dummy;
	struct file_operations *fops;
	int ret;

	ret = prepare_dummy_file(handle, &dummy, filp);
	if (ret)
		return ret;
	fops = dummy.f_dentry->d_inode->i_fop;
	if (fops && fops->release)
		ret = fops->release(inode, &dummy);
	collect_dummy_file(&dummy, filp);
	return ret;
}

int
mfs_low_readdir(mfs_handle_t handle, struct file *filp, void *dirent,
				filldir_t filldir, int *packets, int *bytes)
{
	int ret;
	struct file_operations *fops;
	struct file dummy;
	struct mfs_local_readdir_callback lrb;

	if((ret = prepare_dummy_file(handle, &dummy, filp)))
		return(ret);
	fops = dummy.f_dentry->d_inode->i_fop;
	lrb.realfilldir = filldir;
	lrb.realdirent = dirent;
	lrb.parent = dummy.f_dentry;
	lrb.pmnt = dummy.f_vfsmnt;
	lrb.packets = lrb.bytes = lrb.inbytes = 0;
	ret = vfs_readdir(&dummy, mfs_local_filldir, (void *)&lrb);
	collect_dummy_file(&dummy, filp);
	*packets = lrb.packets;
	*bytes = lrb.bytes;
	return(ret);
}

int
mfs_ioctl(struct inode *ip, struct file *filp, unsigned int cmd, unsigned long arg)
{
	int where;
	mfs_handle_t handle;
	int size;
	int data;
	int ret;

	/* currently supporting only EXT2 ioctls: the following others
	 * are possible candidates (later): ncpfs, smbfs, umsdos */
	switch(cmd)
	{
		case EXT2_IOC_GETFLAGS:
		case EXT2_IOC_GETVERSION:
			size = -(int)sizeof(int);
			break;
		case EXT2_IOC_SETFLAGS:
		case EXT2_IOC_SETVERSION:
			size = sizeof(int);
			break;
		default:
			return(-EPERM);
	}
	if(size > 0 && copy_from_user((void *)&data, (void *)arg, size))
		return(-EFAULT);
	mfs_get_handling(filp->f_dentry, &where, &handle);
	mfs_count_ioctl(where);
	if(where != MFS_PE || mfs_loopback_test)
		ret = mfs_client_ioctl(where, handle, filp, cmd, arg, size,
									&data);
	else
		ret = mfs_low_ioctl(handle, filp, cmd, arg, size, &data);
	if(!ret && size < 0 && copy_to_user((void *)arg, (void *)&data, -size))
		ret = -EFAULT;
	return(ret);
}

int
mfs_low_ioctl(mfs_handle_t handle, struct file *filp, unsigned int cmd,
					unsigned long arg, int size, int *data)
{
	int ret;
	struct file_operations *fops;
	struct file dummy;
	mm_segment_t old_fs;

	if((ret = prepare_dummy_file(handle, &dummy, filp)))
		return(ret);
	fops = dummy.f_dentry->d_inode->i_fop;
	if(!fops || !fops->ioctl)
		ret = -ENOTTY;
	else
	{
		if(size)
			old_fs = set_fs(KERNEL_DS);
		lock_kernel();
		ret = fops->ioctl(dummy.f_dentry->d_inode, &dummy, cmd,
			size ? (unsigned long)data : arg);
		unlock_kernel();
		if(size)
			set_fs(old_fs);
	}
	collect_dummy_file(&dummy, filp);
	return(ret);
}

int
mfs_mmap(struct file *f, struct vm_area_struct *x)
{
	/* cannot be reached: just a place-holder */
	/* some code (including "binfmt_elf.c") check for the existance of this
	 * routine, but "do_mmap" will detect that there are no a_ops
	 * and convert it to a file-read
	 */
	panic("mfs_mmap");
	return(-ENOSYS);
}

int
mfs_fsync(struct file *filp, struct dentry *dp, int datasync)
{
	int where;
	mfs_handle_t handle;

	mfs_get_handling(dp, &where, &handle);
	mfs_count_fsync(where);
	if(where != MFS_PE || mfs_loopback_test)
		return(mfs_client_fsync(where, handle, filp, datasync));
	return(mfs_low_fsync(handle, filp, datasync));
}

int
mfs_low_fsync(mfs_handle_t handle, struct file *filp, int datasync)
{
	int ret;
	struct file_operations *fops;
	struct file dummy;

	if((ret = prepare_dummy_file(handle, &dummy, filp)))
		return(ret);
	fops = dummy.f_dentry->d_inode->i_fop;
	down(&dummy.f_dentry->d_inode->i_sem);
	if(!fops || !fops->fsync)
		ret = -EINVAL;
	else
		ret = fops->fsync(&dummy, dummy.f_dentry, datasync);
	up(&dummy.f_dentry->d_inode->i_sem);
	collect_dummy_file(&dummy, filp);
	return(ret);
}

static struct file_operations mfs_file_file_operations =
{
	llseek:		mfs_file_llseek,
	open:		mfs_open,
	release:	mfs_close,
	read:		mfs_read,
	write:		mfs_write,
	ioctl:		mfs_ioctl,
	mmap:		mfs_mmap,
	fsync:		mfs_fsync,
};

static struct file_operations mfs_symlink_file_operations =
{
	ioctl:		mfs_ioctl,
};

static struct file_operations mfs_dir_file_operations =
{
	llseek:		mfs_dir_llseek,
	open:		mfs_open,
	release:	mfs_close,
	read:		mfs_read,
	readdir:	mfs_readdir,
	ioctl:		mfs_ioctl,
	fsync:		mfs_fsync,
};

struct mfs_iphand
{
	int pe;
	mfs_handle_t handle;
};

int
compare_mfs_ip(struct inode *ip, unsigned long ig1, void *arg)
{
	int pe = ((struct mfs_iphand *)arg)->pe;

	return(!pe || (ip->u.mfs_i.pe == pe &&
		ip->u.mfs_i.handle == ((struct mfs_iphand *)arg)->handle));
}

struct inode *mfs_iget(unsigned long ino, int pe, mfs_handle_t handle)
{
	struct mfs_iphand h;

	h.pe = pe;
	h.handle = handle;
	current->mosix.iget_mfs_pe = pe;
	current->mosix.iget_mfs_handle = handle;
	return(iget4(mfs_super, ino, compare_mfs_ip, &h));
}

int
mfs_revalidate(struct dentry *dentry)
{
	int where;
	mfs_handle_t handle;
	int ret;
	struct subinode iinfo;
	struct inode *ip = dentry->d_inode;

	mfs_get_handling(dentry, &where, &handle);
	mfs_ip_to_subip(&iinfo, ip);
	mfs_count_revalidate(where);
	if(where != MFS_PE || mfs_loopback_test)
		ret = mfs_client_revalidate(where, handle, &iinfo);
	else
		ret = mfs_low_revalidate(handle, &iinfo);
	if(!ret)
		mfs_subip_to_ip(ip, &iinfo);
	return(ret);
}

int
mfs_low_revalidate(mfs_handle_t handle, struct subinode *iinfo)
{
	struct inode *lip;
	int ret;
	struct nameidata nd;

	if((ret = mfs_to_local(handle, &nd)))
		return(ret);
	lip = nd.dentry->d_inode;
	if(!lip || !lip->i_op || !lip->i_op->revalidate)
		ret = 0;
	else
		ret = lip->i_op->revalidate(nd.dentry);
	if(!ret)
		mfs_ip_to_subip(iinfo, lip);
	mfs_lput(nd.dentry, nd.mnt);
	return(ret);
}

int
mfs_setattr(struct dentry *dentry, struct iattr *attr)
{
	int where;
	mfs_handle_t handle;

	if(mfs_is_ro)
		return(-EROFS);
	if(!dentry->d_fsdata) /* someone trying to fiddle with the MFS root */
		return(-EPERM);
	mfs_get_handling(dentry, &where, &handle);
	mfs_count_setattr(where);
	if(where != MFS_PE || mfs_loopback_test)
		return(mfs_client_setattr(where, handle, attr));
	return(mfs_low_setattr(handle, attr));
}

int
mfs_low_setattr(mfs_handle_t handle, struct iattr *attr)
{
	struct nameidata nd;
	struct inode *ip;
	int ret;
	unsigned int wa = 0;

	if((ret = mfs_to_local(handle, &nd)))
		return(ret);
	ret = -EINVAL;
	if(!(ip = nd.dentry->d_inode))
		goto out;
	if(attr->ia_valid & ATTR_SIZE)
	{
		/* this must be ftruncate: */

		if((ret = get_write_access(ip)))
			goto out;
		wa = 1;
		ret = -EISDIR;
		if(S_ISDIR(ip->i_mode))
			goto out;
		ret = -EINVAL;
		if(!S_ISREG(ip->i_mode))
			goto out;
		ret = -EPERM;
		if(IS_APPEND(ip))
			goto out;
		if((ret = locks_verify_truncate(ip, NULL, attr->ia_size)))
			goto out;
		down(&ip->i_sem);
	}
	ret = notify_change(nd.dentry, attr);
	if(attr->ia_valid & ATTR_SIZE)
		up(&ip->i_sem);
	out:
	if(wa)
		put_write_access(ip);
	mfs_lput(nd.dentry, nd.mnt);
	return(ret);
}

int
mfs_root_name_to_ino(char *name, int len)
{
	int node = 0;

	if(*name >= '0' && *name <= '9')
	while(len && node <= MOSIX_MAX && *name >= '0' && *name <= '9')
	{
		node = node * 10 + (*name - '0');
		name++;
		len--;
	}
	else if(len)
	switch(*name)
	{
		case 'h':
			if(len != 4 || name[3] != 'e')
				return(0);
			if(name[1] == 'e' && name[2] == 'r')
				return(current->mosix.whereami ? : PE);
			if(name[1] == 'o' && name[2] == 'm')
				return(current->mosix.deppe ? : PE);
			break;
		case 'l':
			if(len == 8 && !memcmp(&name[1], "astexec", 7))
				return(current->mosix.lastexec ? :
					(current->mosix.deppe ? : PE));
			break;
		case 'm':
			if(len == 5 && !memcmp(&name[1], "agic", 4))
				return(current->mosix.lastmagic ? :
					(current->mosix.deppe ? : PE));
		case 's':
			if(len == 8 && !memcmp(&name[1], "elected", 7))
				return(current->mosix.selected ? :
					(current->mosix.deppe ? : PE));
	}
	if(len || !mos_to_net(node, NULL))
		node = 0;
	return(node);
}

static void
fill_last_component(char *nam, struct nameidata *nd)
{
	char *start = nam, *end;

	while(*nam)
		nam++;
	end = nam;
	for(nam-- ; *nam == '/' && nam > start ; nam--)
		;
	while(nam > start && *(nam-1) != '/')
		nam--;
	nd->last.name = nam;
	nd->last.len = end - nam + 1;
	nd->last_type = LAST_NORM;
	switch(*nam)
	{
		case '/':
		case '\0':
			nd->last_type = LAST_ROOT;
			break;
		case '.':
			switch(*(nam+1))
			{
				case '.':
					if(*(nam+2) == '/' || *(nam+2) == '\0')
						nd->last_type = LAST_DOTDOT;
					break;
				case '/':
				case '\0':
					nd->last_type = LAST_DOT;
					return;
			}
			break;
	}
}

int
mfs_express_lookup(struct nameidata *nd, char **name)
{
	char *nam = *name;
	struct vfsmount *mnt;
	int pe;
	mfs_handle_t handle;
	struct subinode iinfo;
	struct inode *ip;
	struct qstr qnam;
	char *newname = NULL;
	struct mfs_completion complete;
	struct dentry *result;
	int err;
	unsigned int magic = 0;
	int link = 0;
#ifdef CONFIG_MOSIX_DFSA
	unsigned int flags = nd->flags & ~(LOOKUP_COMPLETE|LOOKUP_STAYFS);
#else
	unsigned int flags = nd->flags & ~LOOKUP_COMPLETE;
#endif /* CONFIG_MOSIX_DFSA */

	mfs_prepare_completion(&complete, nd);
	if(complete.error < 0)
	{
		mfs_check_completion(&complete, nd);
		return(complete.error);
	}
	pe = complete.base_pe;
	handle = complete.base;
	*name = NULL;
	while(*nam)
	{
		if(pe == 0)
		{
			char *c;

			if(*nam == '.')
			switch(*(nam+1))
			{
				case '\0':
					nam++;
					continue;
				case '/':
					nam += 2;
					while(*nam == '/')
						nam++;
					continue;
				case '.':
					if(*(nam+2) == '\0' || *(nam+2) == '/')
						goto name_leads_back;
					break;
			}
			for(c = nam ; *c && *c != '/' ; c++)
				;
			pe = mfs_root_name_to_ino(nam, c - nam);
			handle = pe;
			if(!pe)
			{
				mfs_check_completion(&complete, nd);
#ifdef CONFIG_MOSIX_DFSA
				if(current->mosix.dflags & DREMOTE)
					return(-EDOITATHOME);
#endif /* CONFIG_MOSIX_DFSA */
				return(-ENOENT);
			}
			if(nam[0] == 'm')
				magic = 1;
			link = (nam[0] >= 'a');
			while(*c == '/')
				c++;
			nam = c;
		}
		if(flags & LOOKUP_MFS)
		{
			/* server may only access the MFS root itself or reach
			 * back to the node's root via the correct link */
			mfs_check_completion(&complete, nd);
			if(pe != MFS_PE || !(flags & LOOKUP_MFS_MFS))
				return(-EACCES);
			nd->flags &= ~LOOKUP_MFS_MFS;
			dput(nd->dentry);
			mntput(nd->mnt);
			nd->dentry = dget(mfs_root);
			nd->mnt = mntget(mfs_mnt);
			if(*nam)
				*name = nam;
			return(0);
		}
		if(!*nam && (nd->flags & LOOKUP_PARENT))
		{
			mfs_check_completion(&complete, nd);
			return(-EPERM);
		}
		if(pe && !*nam && link && !(nd->flags & LOOKUP_FOLLOW))
			return(mfs_link_complete(&complete, pe, nd));
		mfs_count_express(pe, complete.more_data_len);
		if(pe != MFS_PE || mfs_loopback_test)
			err = mfs_client_express_lookup(pe, &handle, nam,
					flags, &iinfo, &newname, &complete);
		else
			err = mfs_low_express_lookup(&handle, nam, flags,
					current->link_count,
					current->total_link_count, &iinfo,
					&newname, &complete);
#if 0
printk("mfs_express_lookup: nam=<%s>, newname=%x", nam, newname);
if(newname)printk("<%s>\n", newname);else printk("\n");
#endif

		if(err == AN_IMPOSSIBLE_ERROR && iinfo.si_count)
		{
			pe = magic = 0;
#if 0
printk("mfs_express_lookup: skipping from <%s> to ", nam);
#endif
			nam += iinfo.si_count - 1;
#if 0
printk("<%s>\n", nam);
#endif
			if(!*nam && (nd->flags & LOOKUP_PARENT))
			{
				mfs_check_completion(&complete, nd);
				return(-EPERM);
			}
			dput(nd->dentry);
			nd->dentry = dget(mfs_super->s_root);
		}
		else if(IS_ERR((const void *)err))
		{
			mfs_check_completion(&complete, nd);
			return(err);
		}
		else
		{
			if(nd->flags & LOOKUP_PARENT)
				fill_last_component(nam, nd);
			break;
		}
	}
	if(pe == 0)
	{
		mfs_check_completion(&complete, nd);
		return(0);
	}
	if(mfs_check_completion(&complete, nd))
	{
		if(newname)
			kfree(newname);
		return(complete.error);
	}
	if(newname)
	{
		qnam.name = newname;
		qnam.len = strlen(newname);
	}
	else	/* can still function - except 'getcwd' */
	{
		qnam.name = "__removed__";
		qnam.len = 11;
	}
	result = d_alloc(mfs_super->s_root, &qnam);
	if(newname)
		kfree(newname);
	if(!result)
	{
		mfs_dispose_handle(pe, handle);
		return(-ENOMEM);
	}
	if(!(result->d_fsdata = kmalloc(sizeof(struct mfs_dinfo), GFP_KERNEL)))
	{
		dput(result);
		mfs_dispose_handle(pe, handle);
		return(-ENOMEM);
	}
	result->d_flags |= DCACHE_NO_CACHE;
	result->d_op = &mfs_dentry_operations;
	((struct mfs_dinfo *)result->d_fsdata)->pe = pe;
	((struct mfs_dinfo *)result->d_fsdata)->handle = handle;
	((struct mfs_dinfo *)result->d_fsdata)->latest = jiffies;
	if(iinfo.si_ino)
	{
		ip = mfs_iget(iinfo.si_ino, pe, handle);
		if(!ip)
		{
			dput(result);
			return(-EACCES);
		}
		mfs_subip_to_ip(ip, &iinfo);
		d_add(result, ip);
	}
	else
		d_add(result, NULL);
	if(magic && (nd->complete_flags & COMPLETE_DONE))
		nd->complete_flags |= COMPLETE_MAGICAL;
	dput(nd->dentry);
	nd->dentry = result;
	return(0);

	name_leads_back:
	mfs_check_completion(&complete, nd);
#ifdef CONFIG_MOSIX_DFSA
	if(nd->flags & LOOKUP_STAYFS)
	{
		if(current->mosix.dflags & DREMOTE)
			return(-EDOITATHOME);
		nd->flags &= ~LOOKUP_STAYFS;
		current->mosix.dflags &= ~DSTATSDOWN;
	}
#endif /* CONFIG_MOSIX_DFSA */
	for(nam += 2 ; *nam == '/' ; nam++)
		;
#if 0
printk("name leads back, left=../<%s>\n", nam);
#endif
	if(*nam)
		*name = nam;
	else if(nd->flags & LOOKUP_PARENT)
		return(-EPERM);
	dput(nd->dentry);
	mnt = nd->mnt;
	nd->mnt = mntget(mnt->mnt_parent);
	nd->dentry = dget(mnt->mnt_mountpoint->d_parent);
	mntput(mnt);
	return(0);
}

char *
mfs_full_name(struct dentry *of, struct vfsmount *on)
{
	char *a, *b;
	char *page = (char *)kmalloc(PAGE_SIZE, GFP_KERNEL);
	int x;

	if(!page)
		return(NULL);
	if(of == mfs_root && on == mfs_mnt)
	{
		a = &page[PAGE_SIZE-1];
		*a = '\0';
	}
	else
	{
		spin_lock(&dcache_lock);
		a = __d_path(of, on, mfs_root, mfs_mnt, page, PAGE_SIZE);
		spin_unlock(&dcache_lock);
	}
	for(x = MFS_PE ; x ; x /= 10)
	{
		if(a == page)
		{
			kfree(page);
			return(NULL);
		}
		*(--a) = x % 10 + '0';
	}
	if(a != page)
	{
		for(b = page ; *a ; )
			*b++ = *a++;
		*b = '\0';
		a = page;
	}
	return(a);
}

int
mfs_low_express_lookup(mfs_handle_t *handlep, char *name, unsigned int flags,
	int link_count, int total_link_count, struct subinode *iinfo,
	char **newname, struct mfs_completion *complete)
{
	struct inode *ip;
	struct nameidata nd;
	int error;

	iinfo->si_count = 0;
	*newname = NULL;
	if((error = mfs_to_local(*handlep, &nd)))
	{
		mfs_low_complete_error(complete, error);
		return(error);
	}
#if 0
ip = nd.dentry->d_inode;
printk("mfs_low_express_lookup, original ino=%x/%d, d_count=%d\n", ip->i_dev, ip->i_ino, atomic_read(&nd.dentry->d_count));
#endif
	nd.flags = (flags | LOOKUP_MFS) & ~LOOKUP_MFS_MFS;
	nd.last_type = LAST_ROOT;
	nd.name_left = NULL;
	nd.complete_flags = 0;
	dget(nd.dentry);
	mntget(nd.mnt);
	mfs_lput(nd.dentry, nd.mnt);
	current->link_count = link_count;
	current->total_link_count = total_link_count;
	error = path_walk(name, &nd);
	if(nd.name_left)	/* "/.." */
	{
		iinfo->si_count = (nd.name_left - name) + 1;
#if 0
printk("mfs_low_express_lookup: name_left %x<%s>, orig_name=%x<%s>, skip_factor=%d\n", nd.name_left, nd.name_left, name, name, iinfo->si_count);
#endif
		return(AN_IMPOSSIBLE_ERROR);
	}
	if(error)
	{
		mfs_low_complete_error(complete, error);
		return(error);
	}
	if((*handlep = mfs_low_complete(complete, &nd)))
	{
		/* need information back */
		if((ip = nd.dentry->d_inode))
		{
			iinfo->si_ino = global_inum(ip);
			mfs_ip_to_subip(iinfo, ip);
		}
		else
			iinfo->si_ino = 0;
		*newname = mfs_full_name(nd.dentry, nd.mnt);
#if 0
printk("mfs_low_express_lookup: dev=%x, ip=%d, name=<%s>, newname=%x", ip ? ip->i_dev : 0, ip ? ip->i_ino : 0, name, *newname);
if(*newname)printk("<%s>", *newname);printk("\n");
#endif
	}
	else
	{
#if 0
printk("mfs_low_express_lookup(%s): all resolved, err=%d", name, complete->error);
if(nd.dentry->d_inode)printk(", dev=%x, ino=%d\n", nd.dentry->d_inode->i_dev, nd.dentry->d_inode->i_ino);
printk("\n");
#endif
		*handlep = 0;
	}
#if 0
ip = nd.dentry->d_inode; printk("before path release, ");
if(ip)printk("ino=%x/%d, ", ip->i_dev, ip->i_ino);
printk("d_count=%d\n", atomic_read(&nd.dentry->d_count));
#endif 
	if(!IS_ERR((const void *)complete->error))
		path_release(&nd);
	return(complete->error);
}

void
mfs_check_path(struct dentry *dp)
{
	int where;
	mfs_handle_t handle;
	char *name;
	unsigned int len;

	if(dp == mfs_super->s_root)
		return;
	if(dp->d_parent != mfs_super->s_root)
	{
		return;
	}
	mfs_get_handling(dp, &where, &handle);
	if(where != MFS_PE || mfs_loopback_test)
		name = mfs_client_check_path(where, handle);
	else
		name = mfs_low_check_path(handle);
	mfs_count_check_path(where, name ? strlen(name)+1 : 0);
	if(!name || !strcmp(name, dp->d_name.name))
		return;

	len = strlen(name);
	if(len <= dp->d_name.len)
	{
		dp->d_name.len = len;
		strcpy((char *)dp->d_name.name, name);
		kfree(name);
	}
	else
	{
		if(dname_external(dp))
			kfree(dp->d_name.name);
		dp->d_name.name = name;
		dp->d_name.len = len;
	}
}

char *
mfs_low_check_path(mfs_handle_t handle)
{
	struct nameidata nd;
	char *ret;

	if(mfs_to_local(handle, &nd))
		return(NULL);
	ret = mfs_full_name(nd.dentry, nd.mnt);
	mfs_lput(nd.dentry, nd.mnt);
	return(ret);
}

struct inode_operations mfs_file_inode_operations = {
        revalidate:	mfs_revalidate,
	setattr:	mfs_setattr,
	check_path:	mfs_check_path,
};

struct inode_operations mfs_symlink_inode_operations = {
        revalidate:	mfs_revalidate,
	setattr:	mfs_setattr,
	check_path:	mfs_check_path,
};

struct inode_operations mfs_dir_inode_operations = {
        revalidate:	mfs_revalidate,
	setattr:	mfs_setattr,
	express_lookup:	mfs_express_lookup,
	check_path:	mfs_check_path,
};

int
mfs_root_readdir(struct file *filp, void *dirent, filldir_t filldir)
{
	int first, last;

	if(filp->f_pos == 0)
	{
		if(filldir(dirent, ".", 1, 0,
				filp->f_dentry->d_inode->i_ino, DT_DIR) < 0)
			return(0);
		filp->f_pos = 1;
	}
	if(filp->f_pos == 1)
	{
		if(filldir(dirent, "..", 2, 1,
				filp->f_dentry->d_inode->i_ino, DT_DIR) < 0)
			return(0);
		filp->f_pos = 2;
	}
	if(filp->f_pos == 2)
	{
		if(filldir(dirent, "here", 4, 2,
				mfs_root_name_to_ino("here", 4), DT_DIR))
			return(0);
		filp->f_pos = 3;
	}
	if(filp->f_pos == 3)
	{
		if(filldir(dirent, "home", 4, 3,
				mfs_root_name_to_ino("home", 4), DT_DIR))
			return(0);
		filp->f_pos = 4;
	}
	if(filp->f_pos == 4)
	{
		if(filldir(dirent, "lastexec", 8, 4,
				mfs_root_name_to_ino("lastexec", 8), DT_DIR))
			return(0);
		filp->f_pos = 5;
	}
	if(filp->f_pos == 5)
	{
		if(filldir(dirent, "magic", 5, 5,
				mfs_root_name_to_ino("magic", 5), DT_DIR))
			return(0);
		filp->f_pos = 6;
	}
	if(filp->f_pos == 6)
	{
		if(filldir(dirent, "selected", 8, 6,
				mfs_root_name_to_ino("selected", 8), DT_DIR))
			return(0);
		filp->f_pos = 7;
	}
	if(filp->f_pos < 7 || filp->f_pos >= 7 + MOSIX_MAX)
		return(0);
	while(scan_mosix_nodes(filp->f_pos - 6, &first, &last))
	while(first <= last)
	{
		char node[6];
		int n = first;
		char *c;

		c = &node[5];
		*c = '\0';
		while(n)
		{
			c--;
			*c = n % 10 + '0';
			n /= 10;
		}
		if(filldir(dirent, c, sizeof(node) - 1 - (c - node),
			first + 6, first, DT_DIR))
				return(0);
		filp->f_pos = ++first + 6;
	}
	return(0);
}

static struct file_operations mfs_root_file_operations =
{
	readdir:	mfs_root_readdir,
};

int
mfs_root_revalidate(struct dentry *dp)
{
	struct inode *ip = dp->d_inode;

	ip->i_mode = S_IFDIR | S_IRUGO | S_IXUGO;
	ip->i_nlink = 1;
	ip->i_uid = 0;
	ip->i_gid = 0;
	ip->i_rdev = 0;
	ip->i_size = 0;
	ip->i_atime = 0;
	ip->i_ctime = 0;
	ip->i_mtime = 0;
	ip->i_blksize = 1024;
	ip->i_blocks = 0;
	return(0);
}

struct dentry *
mfs_root_lookup(struct inode *dir, struct dentry *dentry)
{
	struct mfs_dinfo *info;
	struct inode *inode = NULL;
	int node;

	if(!(node = mfs_root_name_to_ino((char *)dentry->d_name.name,
							dentry->d_name.len)))
	{
#ifdef CONFIG_MOSIX_DFSA
		if(current->mosix.dflags & DREMOTE)
			return(ERR_PTR(-EDOITATHOME));
#endif /* CONFIG_MOSIX_DFSA */
		return(ERR_PTR(-EACCES));
	}
	if(!(info = (struct mfs_dinfo *)kmalloc(sizeof(struct mfs_dinfo),
								GFP_KERNEL)))
		return(ERR_PTR(-ENOMEM));
	info->pe = node;
	info->handle = node;
	info->latest = jiffies;
	dentry->d_flags |= DCACHE_NO_CACHE;
	dentry->d_op = &mfs_dentry_operations;
	dentry->d_fsdata = info;
	inode = mfs_iget(node, node, node);
	if(!inode)
		return(ERR_PTR(-ENOMEM));
	d_add(dentry, inode);
	return(NULL);
}

/* note: most of the following functions are actually required, even though
 * not really permitted on the MFS root itself, because the MFS root is
 * technically the parent of all MFS nodes.
 */
struct inode_operations mfs_root_inode_operations = {
	revalidate:	mfs_root_revalidate,
	setattr:	mfs_setattr,
	express_lookup:	mfs_express_lookup,
};

void
mfs_read_inode(struct inode * inode)
{
	unsigned long ino = inode->i_ino;
	struct subinode iinfo;
	int err;

	if(ino == MFS_ROOT_INO)
	{
		inode->i_mode = S_IFDIR | S_IXUGO | S_IRUGO | S_IWUGO;
		inode->i_op = &mfs_root_inode_operations;
		inode->i_fop = &mfs_root_file_operations;
		inode->i_mtime = inode->i_atime = inode->i_ctime = CURRENT_TIME;
		inode->i_blocks = 0;
		inode->i_blksize = 1024;
		inode->i_uid = 0;
		inode->i_gid = 0;
		inode->i_nlink = 1;
		inode->i_size = 0;
		inode->i_generation = 0;
		inode->i_version = 0;
		return;
	}
	inode->u.mfs_i.pe = current->mosix.iget_mfs_pe;
	inode->u.mfs_i.handle = current->mosix.iget_mfs_handle;
	if(ino < MFS_ROOT_INO)
	{
		mfs_count_revalidate(ino);
		if(ino != MFS_PE || mfs_loopback_test)
			err = mfs_client_revalidate(ino, ino, &iinfo);
		else
			err = mfs_low_revalidate(MFS_PE, &iinfo);
		if(err)		/* just guess */
		{
			inode->i_mode = S_IFDIR | S_IXUGO | S_IRUGO;
			inode->i_op = &mfs_dir_inode_operations;
			inode->i_mtime = inode->i_atime = inode->i_ctime =
								CURRENT_TIME;
			inode->i_blocks = 0;
			inode->i_blksize = 1024;
			inode->i_uid = 0;
			inode->i_gid = 0;
			inode->i_nlink = 1;
			inode->i_size = 0;
		}
		else
			mfs_subip_to_ip(inode, &iinfo);
	}
}
int mfs_bh_statfs(mfs_handle_t handle, struct statfs *buf)
{

	struct nameidata nd;
	int rc;
	if ((rc = mfs_to_local(handle, &nd))) return (rc);
	rc = vfs_statfs(nd.dentry->d_inode->i_sb, buf);
	mfs_lput(nd.dentry, nd.mnt); /* mfs long put */ 
	return rc;
}	



int
mfs_statfs(struct super_block *sb, struct statfs *buf)
{
	struct dentry *dp; /* use the dentry */
	mfs_handle_t handle; /* get a handle to the mfs fs*/
	unsigned int dest;

	if (current->mosix.dirty_bits & MFSARG_STATFS_FP)
	{
		current->mosix.dirty_bits &= ~MFSARG_STATFS_FP; /* get the file handle */
		dp = current->mosix.dirty_file->f_dentry; /* get the right dentry pointer */
		if (!dp->d_fsdata) goto ESC;
		mfs_get_handling(dp, &dest, &handle); /* get handling and put in dest */
		mfs_count_statfs(dest); /* now actually do it */
		if (dest != MFS_PE)
			return(mfs_client_statfs(dest, handle, buf));
		return (mfs_bh_statfs(handle, buf));
	}

ESC:	
	buf->f_type = MFS_SUPER_MAGIC; /* here we say it's MFS */
	buf->f_bsize = PAGE_SIZE/sizeof(long);
	buf->f_namelen = NAME_MAX;
	buf->f_blocks = 9999999;/* here we should call the concerned PE and get the real values*/
	buf->f_bfree = 99999999;
        buf->f_bavail = 9999999;
        buf->f_files = 9999999;
        buf->f_ffree = 99999;
	return(0);
}

void
mfs_put_super(struct super_block *sb)
{
	mfs_is_mounted = 0;
	mfs_super = NULL;
}

#ifdef CONFIG_MOSIX_DFSA
struct mfs_ident	/* used in mfs_identify/mfs_reconstruct */
{
	int pe;
	mfs_handle_t handle;
	ino_t ino;
	mode_t mode;
};

int
mfs_identify(struct dentry *dentry, void *into)
{
	struct mfs_ident *i = into;
	struct mfs_dinfo *p = (struct mfs_dinfo *)dentry->d_fsdata;

	if(sizeof(struct mfs_ident) > MAX_IDENT_RECORD_LEN)
		panic("mfs_identify: insufficient\n");
	if(p)
	{
		i->pe = p->pe;
		i->handle = p->handle;
	}
	else
	{
		i->pe = 0;
		i->handle = 0;
	}
	i->ino = dentry->d_inode->i_ino;
	i->mode = dentry->d_inode->i_mode;
	return(sizeof(struct mfs_ident));
}

struct dentry *
mfs_reconstruct(struct vfsmount *mnt, void *info, int infolen)
{
	struct mfs_ident *i = (struct mfs_ident *)info;
	struct dentry *dp;
	struct inode *ip;
	struct qstr qname;

	if(mnt->mnt_sb != mfs_super || infolen != sizeof(struct mfs_ident))
	{
		printk("mfs_reconstruct: improper data\n");
		return(NULL);
	}
	if(!i->pe)
		return(i->handle ? NULL : dget(mfs_super->s_root));
	qname.name = "yet_unknown";
	qname.len = 11;
	if(!(dp = d_alloc(mfs_super->s_root, &qname)))
		return(NULL);
	if(!(dp->d_fsdata = kmalloc(sizeof(struct mfs_dinfo), GFP_KERNEL)))
	{
		dput(dp);
		return(NULL);
	}
	dp->d_flags |= DCACHE_NO_CACHE;
	dp->d_op = &mfs_dentry_operations;
	((struct mfs_dinfo *)dp->d_fsdata)->pe = i->pe;
	((struct mfs_dinfo *)dp->d_fsdata)->handle = i->handle;
	((struct mfs_dinfo *)dp->d_fsdata)->latest = jiffies;
	ip = mfs_iget(i->ino, i->pe, i->handle);
	if(!ip)
	{
		kfree(dp->d_fsdata);
		dp->d_fsdata = NULL;
		dput(dp);
		return(NULL);
	}
	ip->i_mode = i->mode;
	switch(i->mode & S_IFMT)
	{
		case S_IFREG:
			ip->i_op = &mfs_file_inode_operations;
			ip->i_fop = &mfs_file_file_operations;
			break;
		case S_IFLNK:
			ip->i_op = &mfs_symlink_inode_operations;
			ip->i_fop = &mfs_symlink_file_operations;
			break;
		case S_IFDIR:
			ip->i_op = &mfs_dir_inode_operations;
			ip->i_fop = &mfs_dir_file_operations;
			break;
		default:
			printk("mfs_reconstruct: inum %d of bad mode (0%o)\n",
						(int)ip->i_ino, ip->i_mode);
			make_bad_inode(ip);
			break;
	}
	d_add(dp, ip);
	if(mfs_attach_handle(i->pe, i->handle))
	{
		kfree(dp->d_fsdata);
		dp->d_fsdata = NULL;
		dput(dp);
		return(NULL);
	}
	return(dp);
}

void
mfs_dfsa_changed(struct vfsmount *mnt, int new)
{
	mfs_is_dfsa = (new != 0);
}
#endif /* CONFIG_MOSIX_DFSA */

struct super_operations mfs_sops =
{
	read_inode:		mfs_read_inode,
	put_inode:		force_delete,
	put_super:		mfs_put_super,
	statfs:			mfs_statfs,
#ifdef CONFIG_MOSIX_DFSA
	identify:		mfs_identify,
	reconstruct:		mfs_reconstruct,
	dfsa_changed:		mfs_dfsa_changed,
#endif /* CONFIG_MOSIX_DFSA */
};

int
mfs_dentry_compare(struct dentry *d, struct qstr *a, struct qstr *b)
{
	/* since MFS uses no hashing/caching, we should never reach here! */
	printk("mfs_dentry_compare! (<%s>, <%s>)\n", a->name, b->name);
	return(1);
}

void
mfs_dentry_release(struct dentry *dp)
{
	int where;
	mfs_handle_t handle;

	if(dp->d_fsdata)
	{
		where = ((struct mfs_dinfo *)dp->d_fsdata)->pe;
		handle = ((struct mfs_dinfo *)dp->d_fsdata)->handle;
		kfree(dp->d_fsdata);
		dp->d_fsdata = NULL;
		mfs_dispose_handle(where, handle);
	}
}

struct dentry_operations mfs_dentry_operations =
{
	d_compare:		mfs_dentry_compare,
	d_release:		mfs_dentry_release,
};

struct super_block *
mfs_read_super(struct super_block *s, void *data, int silent)
{
	struct inode * root_inode;
	struct super_block *ret = s;

	if(mfs_is_mounted)
	{
		printk("MFS already mounted!\n");
		return(NULL);
	}
	if(!PE)
		printk("WARNING: MFS mounting is ineffective until openMosix is configured.\n");
	mfs_is_mounted = 1;
	mfs_is_ro = (s->s_flags & MS_RDONLY);
	mfs_super = s;
	s->s_blocksize = 1024;
	s->s_blocksize_bits = 10;
	s->s_magic = MFS_SUPER_MAGIC;
	s->s_op = &mfs_sops;
	root_inode = iget(s, MFS_ROOT_INO);
	if (!root_inode || !(s->s_root = d_alloc_root(root_inode)))
	{
		printk("Failed initializing openMosix file-system.\n");
		iput(root_inode);
		mfs_is_mounted = 0;
		ret = NULL;
	}
	else
		s->s_root->d_op = &mfs_dentry_operations;
	s->s_maxbytes = ~0LL;
	return(ret);
}

void
mfs_change_pe(void)
{
	lock_mosix();
	if(MFS_PE == ((PE && !mosadmin_mode_nomfs) ? PE : MFS_UNKNOWN_PE))
	{
		unlock_mosix();
		return;
	}
	unlock_mosix();
	if(mfs_try_to_cleanup(0))
	{
		printk("WARNING: Due to the openMosix configuration-change, all MFS files and\n");
		printk("directories previously opened on this node will become inaccessible!\n");
		mfs_try_to_cleanup(2);
	}
	lock_mosix();
	MFS_PE = (PE && !mosadmin_mode_nomfs) ? PE : MFS_UNKNOWN_PE;
	unlock_mosix();
}

DECLARE_FSTYPE(mfs_fs_type, "mfs", mfs_read_super, 0);

int init_mfs(void)
{
	int err;

	MFS_PE = (PE && !mosadmin_mode_nomfs) ? PE : MFS_UNKNOWN_PE;
	if((err = register_filesystem(&mfs_fs_type)))
		return err;
	mfs_conversion_init();
	mfs_make_servers();
	return 0;
}

void exit_mfs(void)
{
	unregister_filesystem(&mfs_fs_type);
}

int
mfs_walk_init_root(struct nameidata *nd)
{
	int ret;

	lock_kernel();
	if(mfs_root)
	{
		nd->dentry = dget(mfs_root);
		nd->mnt = mntget(mfs_mnt);
		ret = 1;
	}
	else
		ret = 0;
	unlock_kernel();
	return(ret);
}

int
is_mfs_root(struct nameidata *nd)
{
	return(nd->dentry == mfs_root && nd->mnt == mfs_mnt);
}

void
mfs_report_used(void)
{
	struct dentry *root;
	struct dentry *dp;
	struct list_head *next;
	unsigned long timo = jiffies + 10 * HZ;
	struct mfs_dinfo *i;
	
	loop:
	if(!mfs_super)
		return;
	root = mfs_super->s_root;
	dget(root);
	for(next = root->d_subdirs.next ; next != &root->d_subdirs &&
				time_before(jiffies, timo) ; next = next->next)
	{
		dp = list_entry(next, struct dentry, d_child);
		if((i = dp->d_fsdata) &&
			time_before(i->latest + MFS_REPORT_TIME, jiffies))
		{
			dget(dp);
			i->latest = jiffies;
			if(mfs_touch_handle(i->pe, i->handle))
				i->pe = MFS_BAD_PE;
			dput(dp);
			goto loop;
		}
	}
	dput(root);
}

int
mfs_client_daemon(void *nothing)
{
	struct task_struct *p = current;

	lock_kernel();
	common_daemon_setup("oMFS_gc", 0);
	while(1)
	{
		mfs_report_used();
		mfs_monitor_client_contacts();
		p->state = TASK_INTERRUPTIBLE;
		schedule_timeout(20 * HZ);
		flush_signals(p);
	}
}
#endif /* CONFIG_MOSIX_FS */
