/* All changes since Feb 12, 2002 copyright by Moshe Bar (moshe@moshebar.com)
 * Copyright (C) 2000, 2001, Amnon Barak (amnon@cs.huji.ac.il)
 *
 * Permission to use this software is hereby granted under the terms of the
 * GNU General Public License, as published by the Free Software Foundation.
 *
 * THIS  SOFTWARE  IS  PROVIDED IN ITS  "AS IS"  CONDITION, WITH NO WARRANTY
 * WHATSOEVER. NO LIABILITY OF ANY KIND FOR ANY DAMAGES WHATSOEVER RESULTING
 * FROM THE USE OF THIS SOFTWARE WILL BE ACCEPTED.
 */
/*
 * Author(s):Moshe Bar,  Amnon Shiloh.
 */

#include <linux/sched.h>
#ifdef CONFIG_MOSIX_FS
#include <linux/slab.h>
#include <linux/mfs.h>
#include <linux/dirent.h>

void
unpack_credits(struct mfs_cred *cred)
{
	register struct task_struct *p = current;
	register int i;

	p->uid = cred->uid;
	p->euid = cred->euid;
	p->sgid = cred->sgid;
	p->fsuid = cred->fsuid;
	p->gid = cred->gid;
	p->egid = cred->egid;
	p->sgid = cred->sgid;
	p->fsgid = cred->fsgid;
	p->ngroups = cred->ngroups;
	for(i = cred->ngroups-1 ; i >= 0 ; i--)
		p->groups[i] = cred->groups[i];
	p->cap_effective = (p->cap_effective & ~REMOTE_CAPS) |
						(cred->caps & REMOTE_CAPS);
	p->mosix.whereami = cred->whereami;
	p->mosix.deppe = cred->deppe;
}
/* Ported to oM by Moshe Bar */ 
void
mfs_serve_statfs(struct mfs_server_contact *con, struct mfs_request_statfs *s)

{
	struct mfs_reply_statfs r; /* by Shiloh and Moshe Bar*/
	unpack_credits(&s->cred);
	r.result = mfs_bh_statfs(s->handle, &r.stat); /*run bottom half part of syscall */
	 mfs_server_send(con, MFS_REPLY_STATFS, &r, sizeof(r), NULL, 0,NULL, 0);
}


void
mfs_serve_attach(struct mfs_server_contact *con,
					struct mfs_request_attach_handle *s)
{
	struct mfs_reply_attach_handle r;

	r.result = mfs_low_attach_handle(s->handle);
	mfs_server_send(con, MFS_REPLY_ATTACH_HANDLE, &r, sizeof(r), NULL, 0,
								NULL, 0);
}

void
mfs_serve_touch(struct mfs_server_contact *con,
					struct mfs_request_touch_handle *s)
{
	struct mfs_reply_touch_handle r;

	r.result = mfs_low_touch_handle(s->handle);
	mfs_server_send(con, MFS_REPLY_TOUCH_HANDLE, &r, sizeof(r), NULL, 0,
								NULL, 0);
}

void
mfs_serve_open(struct mfs_server_contact *con, struct mfs_request_open *s)
{
	struct mfs_reply_open r;

	unpack_credits(&s->cred);
	r.result = mfs_low_open(s->handle, &s->inode, &s->file);
	r.inode = s->inode;
	r.file = s->file;
	mfs_server_send(con, MFS_REPLY_OPEN, &r, sizeof(r), NULL, 0, NULL, 0);
}

void
mfs_serve_close(struct mfs_server_contact *con, struct mfs_request_open *s)
{
	struct mfs_reply_open r;

	unpack_credits(&s->cred);
	r.result = mfs_low_close(s->handle, &s->inode, &s->file);
	r.inode = s->inode;
	r.file = s->file;
	mfs_server_send(con, MFS_REPLY_CLOSE, &r, sizeof(r), NULL, 0, NULL, 0);
}

void
mfs_serve_llseek(struct mfs_server_contact *con, struct mfs_request_llseek *s)
{
	struct mfs_reply_llseek r;

	unpack_credits(&s->cred);
	r.result = mfs_low_llseek(s->handle, &s->file, s->offset, s->origin);
	r.file = s->file;
	mfs_server_send(con, MFS_REPLY_LLSEEK, &r, sizeof(r), NULL, 0, NULL, 0);
}

/* we also need to do read-ahead soon Moshe Bar*/
void
mfs_serve_read(struct mfs_server_contact *con, struct mfs_request_read *s)
{
	struct mfs_reply_read r;
	int n;
	size_t count = s->count;
	size_t cnt, sent = 0;
	char *buf = NULL;
	int ret = 0;
	size_t max;
	char smallbuf[4];
	struct mfs_interim_data d;

	for(max = s->count > 16384 ? 16384 : s->count ; max > sizeof(smallbuf) ;
								max >>= 1)
	if((buf = (char *)kmalloc(max, GFP_KERNEL)))
		break;
	if(max <= sizeof(smallbuf))
	{
		buf = smallbuf;
		max = sizeof(smallbuf);
	}
	unpack_credits(&s->cred);
	while(count > 0)
	{
		cnt = count;
		if(cnt > max)
			cnt = max;
		n = mfs_low_read(s->handle, &s->file, buf, cnt, &s->pos);
		if(n < cnt || cnt == count)
		{
			if(n < 0)
				ret = n;
			else
				ret += n;
			break;
		}
		ret += cnt;
		d.count = cnt;
		if(mfs_server_send(con, MFS_INTERIM_DATA, (void *)&d, sizeof(d),
			(void *)buf, cnt, NULL, 0))
			goto free_buf;
		sent += cnt;
		count -= cnt;
	}
	r.file = s->file;
	r.ppos = s->pos;
	r.result = ret;
	r.datalen = (ret > 0 && ret > sent) ? ret - sent : 0;
	mfs_server_send(con, MFS_REPLY_READ, &r, sizeof(r),
				r.datalen ? buf : NULL, r.datalen, NULL, 0);
	free_buf:
	if(max != sizeof(smallbuf))
		kfree(buf);
}

void
mfs_serve_write(struct mfs_server_contact *con, struct mfs_request_write *s,
								void *buf)
{
	struct mfs_reply_write r;
	struct mfs_interim_data *i = NULL;
	ssize_t n;
	size_t count = s->count;
	size_t cnt;
	struct task_struct *p = current;

	unpack_credits(&s->cred);
	if(signal_pending(p))
	{
		spin_lock_irq(&p->sigmask_lock);
		flush_signals(p);
		spin_unlock_irq(&p->sigmask_lock);
	}
	cnt = s->datalen;
	if(cnt < count)
		mfs_server_send(con, MFS_CONTINUE, NULL, 0, NULL, 0, NULL, 0);
	r.result = mfs_low_write(s->handle, &s->file, buf, cnt,&s->pos,s->flim);
	/* in Linux, an append-"write", once started is not broken even if
	 * another process interfered */
	s->file.f_flags &= ~O_APPEND;
	if(r.result == cnt)
	while(r.result < count)
	{
		if(i)
			kfree(i);
		if((n = mfs_server_receive(con, (void **)&i, &buf, NULL)) < 0)
			return;
		if(n == MFS_STOP)
		{
			r.result = -EFAULT;
			break;
		}
		if(n != MFS_INTERIM_DATA)
		{
			printk("mfs_serve_write: unexpected type: %d\n", n);
			if(i)
				kfree(i);
			return;
		}
		n = mfs_low_write(s->handle, &s->file, buf, i->count, &s->pos,
								s->flim);
		r.result += n;
		if(n < i->count)
		{
			if(n < 0)
				r.result = n;
			break;
		}
	}
	if(i)
		kfree(i);
	r.file = s->file;
	r.ppos = s->pos;
	if(sigismember(&p->pending.signal, SIGXFSZ))
	{
		spin_lock_irq(&p->sigmask_lock);
		flush_signals(p);
		spin_unlock_irq(&p->sigmask_lock);
		r.hadsigxfsz = 1;
	}
	else
		r.hadsigxfsz = 0;
	current->rlim[RLIMIT_FSIZE].rlim_cur = RLIM_INFINITY;
	mfs_server_send(con, MFS_REPLY_WRITE, &r, sizeof(r), NULL, 0, NULL, 0);
}

struct mfs_serve_readdir_callback
{
	struct mfs_server_contact *con;
	void *buf, *bufp;
	int buflen;
	int policy;
	int maxbytes;
};

int
mfs_serve_flush_dirs(struct mfs_serve_readdir_callback *cb)
{
	struct mfs_interim_data i;
	struct mfs_interim_readdir *ri;
	int t, err;

	i.count = cb->bufp - cb->buf;
	if(i.count == 0)
		return(0);
	if(mfs_server_send(cb->con, MFS_INTERIM_DATA, (void *)&i, sizeof(i),
						cb->buf, i.count, NULL, 0))
		return(-EINVAL);
	t = mfs_server_receive(cb->con, (void **)&ri, NULL, NULL);
	if(t == MFS_INTERIM_READDIR)
		err = ri->error;
	else
		err = -EINVAL;
	if(ri)
		kfree(ri);
	if(err)
		return(err);
	cb->bufp = cb->buf;
	return(0);
}

int
mfs_serve_filldir(void *__cb, const char *name, int namelen, loff_t offset,
						ino_t ino, unsigned int type)
{
	struct mfs_serve_readdir_callback *cb = __cb;
	int reclen;
	int err;

	switch(cb->policy)
	{
		case MFS_READDIR_JUSTONE:
			if(!cb->maxbytes)
				return(-EINVAL);
			cb->maxbytes = 0;
			break;
		case MFS_READDIR_GETDENTS:
			reclen = (offsetof(struct dirent, d_name[0]) + namelen +
				1 + sizeof(long)-1) & ~(sizeof(long)-1);
			if(reclen > cb->maxbytes)
				return(-EINVAL);
			cb->maxbytes -= reclen;
			break;
		case MFS_READDIR_GETDENTS64:
			reclen = (offsetof(struct dirent64, d_name[0]) +
				namelen + 1 + sizeof(u64)-1) & ~(sizeof(u64)-1);
			if(reclen > cb->maxbytes)
				return(-EINVAL);
			cb->maxbytes -= reclen;
			break;
	}
	reclen = sizeof(ino_t) + sizeof(loff_t) + sizeof(unsigned int) +
						sizeof(short) + namelen;
	if(reclen > cb->buflen - (cb->bufp - cb->buf))
	{
		if(cb->bufp == cb->buf || (cb->policy != MFS_READDIR_GETDENTS
				&& cb->policy != MFS_READDIR_GETDENTS64))
		{
			/* should not happen - we allocated at least dirent64 */
			printk("mfs_serve_filldir: buffer not sufficient\n");
			return(-EINVAL);
		}
		if((err = mfs_serve_flush_dirs(cb)))
			return(err);
	}
	*((ino_t *)cb->bufp)++ = ino;
	*((loff_t *)cb->bufp)++ = offset;
	*((unsigned int *)cb->bufp)++ = type;
	*((short *)cb->bufp)++ = namelen;
	memcpy(cb->bufp, (void *)name, namelen);
	cb->bufp += namelen;
	if(cb->policy == MFS_READDIR_UNKNOWN)
		return(mfs_serve_flush_dirs(cb));
	else
		return(0);
}

void
mfs_serve_readdir(struct mfs_server_contact *con, struct mfs_request_readdir *s)
{
	struct mfs_reply_readdir r;
	int ret = 0;
	struct mfs_serve_readdir_callback cb;
	int multi = (s->policy == MFS_READDIR_GETDENTS ||
					s->policy == MFS_READDIR_GETDENTS64);
	int duma, dumb;

	for(cb.buflen = multi ? 16384 : sizeof(struct dirent64) ;
		cb.buflen >= sizeof(struct dirent64) ; cb.buflen >>= 1)
	if((cb.buf = (char *)kmalloc(cb.buflen, GFP_KERNEL)))
		break;
	cb.bufp = cb.buf;
	cb.policy = s->policy;
	cb.maxbytes = multi ? s->maxbytes : 1;
	cb.con = con;
	unpack_credits(&s->cred);
	if(cb.buflen < sizeof(struct dirent64))
		ret = -ENOMEM;
	else
		ret = mfs_low_readdir(s->handle, &s->file, (void *)&cb, 
					mfs_serve_filldir, &duma, &dumb);
	r.file = s->file;
	r.result = ret;
	r.datalen = cb.bufp - cb.buf;
	mfs_server_send(con, MFS_REPLY_READDIR, &r, sizeof(r),
				r.datalen ? cb.buf : NULL, r.datalen, NULL, 0);
	if(cb.buflen >= sizeof(struct dirent64))
		kfree(cb.buf);
}

void
mfs_serve_ioctl(struct mfs_server_contact *con, struct mfs_request_ioctl *s)
{
	struct mfs_reply_ioctl r;

	unpack_credits(&s->cred);
	r.result = mfs_low_ioctl(s->handle, &s->file, s->cmd, s->arg, s->size,
								&s->data);
	r.file = s->file;
	r.data = s->data;
	mfs_server_send(con, MFS_REPLY_IOCTL, &r, sizeof(r), NULL, 0, NULL, 0);
}

void
mfs_serve_fsync(struct mfs_server_contact *con, struct mfs_request_fsync *s)
{
	struct mfs_reply_fsync r;

	unpack_credits(&s->cred);
	r.result = mfs_low_fsync(s->handle, &s->file, s->datasync);
	r.file = s->file;
	mfs_server_send(con, MFS_REPLY_FSYNC, &r, sizeof(r), NULL, 0, NULL, 0);
}

void
mfs_serve_revalidate(struct mfs_server_contact *con,
					struct mfs_request_revalidate *s)
{
	struct mfs_reply_revalidate r;

	unpack_credits(&s->cred);
	r.result = mfs_low_revalidate(s->handle, &r.iinfo);
	mfs_server_send(con, MFS_REPLY_REVALIDATE, &r, sizeof(r), NULL, 0, NULL, 0);
}

void
mfs_serve_express(struct mfs_server_contact *con, struct mfs_request_express *s,
						char *name, void *more_data)
{
	struct mfs_reply_express r;
	char *newname;

	unpack_credits(&s->cred);
	r.handle = s->handle;
	r.complete = s->complete;
	if(r.complete.more_data)
		r.complete.more_data = more_data;
	r.result = mfs_low_express_lookup(&r.handle, name, s->follow,
			s->link_count, s->total_link_count, &r.iinfo,
			&newname, &r.complete);
	r.namelen = (r.result || !newname) ? 0 : strlen(newname) + 1;
	mfs_server_send(con, MFS_REPLY_EXPRESS, &r, sizeof(r),
				r.namelen ? (void *)newname : NULL, r.namelen,
				r.complete.more_data, r.complete.more_data_len);
	if(r.namelen)
		kfree(newname);
	if(r.complete.more_data)
		kfree(r.complete.more_data);
}

void
mfs_serve_checkpath(struct mfs_server_contact *con,
						struct mfs_request_checkpath *s)
{
	struct mfs_reply_checkpath r;
	char *path;

	unpack_credits(&s->cred);
	path = mfs_low_check_path(s->handle);
	r.namelen = path ? strlen(path)+1 : 0;
	mfs_server_send(con, MFS_REPLY_CHECKPATH, &r, sizeof(r),
		r.namelen ? (void *)path : NULL, r.namelen, NULL, 0);
	if(path)
		kfree(path);
}

void
mfs_serve_setattr(struct mfs_server_contact *con, struct mfs_request_setattr *s)
{
	struct mfs_reply_setattr r;

	unpack_credits(&s->cred);
	r.result = mfs_low_setattr(s->handle, &s->attr);
	mfs_server_send(con, MFS_REPLY_SETATTR, &r, sizeof(r), NULL, 0,
								NULL, 0);
}

void
mfs_serve(struct mfs_server_contact *con, int type, void *header, void *dat1,
								void *dat2)
{
	switch(type)
	{
	case MFS_REQUEST_ATTACH_HANDLE:
		mfs_serve_attach(con,
				(struct mfs_request_attach_handle *)header);
		break;
	case MFS_REQUEST_TOUCH_HANDLE:
		mfs_serve_touch(con,(struct mfs_request_touch_handle *)header);
		break;
	case MFS_REQUEST_DISPOSE_HANDLE:
		mfs_low_dispose_handle(
			((struct mfs_request_dispose_handle *)header)->handle);
		mfs_server_send(con, MFS_CONTINUE, NULL, 0, NULL, 0, NULL,0);
		break;
	case MFS_REQUEST_LLSEEK:
		mfs_serve_llseek(con, (struct mfs_request_llseek *)header);
		break;
	case MFS_REQUEST_READ:
		mfs_serve_read(con, (struct mfs_request_read *)header);
		break;
	case MFS_REQUEST_WRITE:
		mfs_serve_write(con, (struct mfs_request_write *)header, dat1);
		break;
	case MFS_REQUEST_OPEN:
		mfs_serve_open(con, (struct mfs_request_open *)header);
		break;
	case MFS_REQUEST_CLOSE:
		mfs_serve_close(con, (struct mfs_request_open *)header);
		break;
	case MFS_REQUEST_READDIR:
		mfs_serve_readdir(con, (struct mfs_request_readdir *)header);
		break;
	case MFS_REQUEST_IOCTL:
		mfs_serve_ioctl(con, (struct mfs_request_ioctl *)header);
		break;
	case MFS_REQUEST_FSYNC:
		mfs_serve_fsync(con, (struct mfs_request_fsync *)header);
		break;
	case MFS_REQUEST_REVALIDATE:
		mfs_serve_revalidate(con,
				(struct mfs_request_revalidate *)header);
		break;
	case MFS_REQUEST_EXPRESS:
		mfs_serve_express(con, (struct mfs_request_express *)header,
							(char *)dat1, dat2);
		break;
	case MFS_REQUEST_CHECKPATH:
		mfs_serve_checkpath(con,
					(struct mfs_request_checkpath *)header);
		break;
	case MFS_REQUEST_SETATTR:
		mfs_serve_setattr(con, (struct mfs_request_setattr *)header);
		break;
	case MFS_REQUEST_STATFS:
		mfs_serve_statfs(con, (struct mfs_request_statfs *)header);
		break;
	default:
		printk("mfs_serve: type %d unimplemented\n", type);
	}
}

#endif /* CONFIG_MOSIX_FS */
